#include "infoled.h"
#include "main.h"
#include "utils/timebase.h"


/**
 * @brief Turn a LED on
 * @param led
 */
void led_on(enum led_enum led)
{
	LED_GPIO->BSRR |= led;
}


/**
 * @brief Turn a LED off
 * @param led
 */
void led_off(enum led_enum led)
{
	LED_GPIO->BRR |= led;
}


/**
 * @brief Toggle a LED state
 * @param led
 */
void led_toggle(enum led_enum led)
{
	LED_GPIO->ODR ^= led;
}



static task_pid_t blink_r_task_id = PID_NONE;
static task_pid_t blink_g_task_id = PID_NONE;


/* Callback */
static void led_blink_r_task(void)
{
	blink_r_task_id = PID_NONE; // mark as ended
	led_off(LED_R);
}


/* Callback */
static void led_blink_g_task(void)
{
	blink_g_task_id = PID_NONE; // mark as ended
	led_off(LED_G);
}


/**
 * @brief Blink a LED (async)
 * @param led
 * @param ms  - blink duration
 */
void led_blink(enum led_enum led, uint32_t ms)
{
	switch (led) {
		case LED_R:
			abort_scheduled_task(blink_r_task_id);

			if ((blink_r_task_id = schedule_task(led_blink_r_task, ms, false))) {
				// only if end task was started
				led_on(LED_R);
			}
			break;

		case LED_G:
			abort_scheduled_task(blink_g_task_id);

			if ((blink_g_task_id = schedule_task(led_blink_g_task, ms, false))) {
				// only if end task was started
				led_on(LED_G);
			}
			break;
	}
}
