#pragma once

/* Includes ------------------------------------------------------------------*/

#include "main.h"


/* Exported types ------------------------------------------------------------*/

typedef enum {
	RST_LPWR = 0x80,
	RST_WWDG = 0x40,
	RST_IWDG = 0x20,
	RST_SOFT = 0x10,
	RST_POR = 0x08,
	RST_PIN = 0x04,
	RST_OBL = 0x02,
	RST_START = 0x0C,
	RST_UNKNOWN = 0x00
} ResetReason;

/* Exported constants --------------------------------------------------------*/
/* Exported macro ------------------------------------------------------------*/
/* Exported functions ------------------------------------------------------- */

/**
 * @brief Get reset reason code (straight from RCC->CSR)
 * @return byte with reset flags (upper byte if CSR)
 */
uint8_t get_reset_reason_raw(void);


/**
 * @brief Get reset reason enum
 * @return reset reason
 */
ResetReason get_reset_reason(void);


/**
 * @brief Get reset reason as a string.
 * @return Short string literal describing the reset reason.
 */
const char *get_reset_reason_str(void);


/**
 * @brief Clear the reset reason flags.
 *
 * This should be called after the reset reason was read out.
 */
void clear_reset_reason(void);


/**
 * @brief Perform a software reset using the SCB
 */
void SW_Reset(void);
